package com.yanzuoguang.util.helper;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

/**
 * 类型帮助类
 *
 * @author 颜佐光
 */
public class TypeHelper {

    /**
     * 　判断是否属于子类
     *
     * @param from
     * @param to
     * @return
     */
    public static boolean isSubType(Type from, Class to) {
        Class returnClass = getClass(from);
        return isSubClass(returnClass, to);
    }

    /**
     * 　判断是否属于子类
     *
     * @param from
     * @param to
     * @return
     */
    public static boolean isSubClass(Class from, Class to) {
        if (from == null) {
            return false;
        }
        if (from.isAssignableFrom(to)) {
            return true;
        }
        return isSubClass(from.getSuperclass(), to);
    }

    /**
     * 获取type的class
     *
     * @param type
     * @return
     */
    public static Class getClass(Type type) {
        if (type instanceof ParameterizedType) {
            Type rawType = ((ParameterizedType) type).getRawType();
            return getClass(rawType);
        } else if (type instanceof Class) {
            return (Class) type;
        }
        return null;
    }

    /**
     * 打印类型
     *
     * @param returnType
     */
    public static void printSuperClass(Class<?> returnType) {
        if (returnType == null) {
            return;
        }
        System.out.println(returnType.getName());
        printSuperClass(returnType.getSuperclass());
    }
}
